#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <mpi.h>
#include <signal.h>
#include <unistd.h>

//Prueba de Metodología de la Programación Paralela
//enero 2019
//Conteo de distancias a elementos iguales en una matriz
//Esquema para entrada/salida, validación y toma de tiempos

void generar(char *m, int t,int numcar) {
  int i;

  for (i = 0; i < t; i++) {
      m[i] =(char) ((int) (((1. * rand()) / RAND_MAX)*numcar)+'a');
  }
}

void escribir(char *m, int N,int M) {
  int i, j;

  for (i = 0; i < N; i++) {
    for (j = 0; j < M; j++)
      printf("%c", m[i * M + j]);
    printf("\n");
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern int sec(int n,char *A,int nodo,int np);

int main(int argc,char *argv[]) {
  int N; //tamaño matriz
  char *A;
  int cuantos; //número de problemas
  int semilla; //semilla para generación aleatoria
  int numcar; //número de caracteres distintos a generar
  int resul; //almacena el resultado de sec
  long long ti,tf,tt=0;
  int nodo,np;

  FILE *stats_file = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(10);  /* time limit */
 
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&np);
  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);
 
// The number of test cases is read
// and sent to all the processes
  if(nodo==0)
  {
    scanf("%d",&cuantos);
    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  else
  {
    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
  }

  for(int i=0;i<cuantos;i++)
  {
    if(nodo==0)
    {
      scanf("%d",&N);                   
      scanf("%d",&semilla);  
      scanf("%d",&numcar); 

      A = (char *) calloc(sizeof(char),N*N);

      srand(semilla);
      generar(A,N*N,numcar);
#ifdef DEBUG
  escribir(A,N,N);
#endif
    }
    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    resul=sec(N,A,nodo,np);
    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
    if(nodo==0)
    {
#ifdef DEBUG
  escribir(A,N,N);
#endif
      printf("%d\n",resul);
      if(i!=0) tt+=tf-ti;
      free(A);
    }
  }
  if(nodo==0)
  {   
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
  }
  MPI_Finalize();
  return 0;
}
